@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 import proto  # type: ignore
 
 from google.cloud.datastore_admin_v1.types import index as gda_index
+from google.cloud.datastore_admin_v1.types import migration
 from google.protobuf import timestamp_pb2  # type: ignore
 
 
@@ -37,6 +38,7 @@ __protobuf__ = proto.module(
         "ListIndexesRequest",
         "ListIndexesResponse",
         "IndexOperationMetadata",
+        "DatastoreFirestoreMigrationMetadata",
     },
 )
 
@@ -62,7 +64,7 @@ class CommonMetadata(proto.Message):
         operation_type (google.cloud.datastore_admin_v1.types.OperationType):
             The type of the operation. Can be used as a
             filter in ListOperationsRequest.
-        labels (Sequence[google.cloud.datastore_admin_v1.types.CommonMetadata.LabelsEntry]):
+        labels (Dict[str, str]):
             The client-assigned labels which were
             provided when the operation was created. May
             also include additional labels.
@@ -113,7 +115,7 @@ class ExportEntitiesRequest(proto.Message):
         project_id (str):
             Required. Project ID against which to make
             the request.
-        labels (Sequence[google.cloud.datastore_admin_v1.types.ExportEntitiesRequest.LabelsEntry]):
+        labels (Dict[str, str]):
             Client-assigned labels.
         entity_filter (google.cloud.datastore_admin_v1.types.EntityFilter):
             Description of what data from the project is
@@ -157,7 +159,7 @@ class ImportEntitiesRequest(proto.Message):
         project_id (str):
             Required. Project ID against which to make
             the request.
-        labels (Sequence[google.cloud.datastore_admin_v1.types.ImportEntitiesRequest.LabelsEntry]):
+        labels (Dict[str, str]):
             Client-assigned labels.
         input_url (str):
             Required. The full resource URL of the external storage
@@ -410,4 +412,27 @@ class IndexOperationMetadata(proto.Message):
     index_id = proto.Field(proto.STRING, number=3,)
 
 
+class DatastoreFirestoreMigrationMetadata(proto.Message):
+    r"""Metadata for Datastore to Firestore migration operations.
+
+    The DatastoreFirestoreMigration operation is not started by the
+    end-user via an explicit "creation" method. This is an intentional
+    deviation from the LRO design pattern.
+
+    This singleton resource can be accessed at:
+    ``projects/{project_id}/datastore-firestore-migration``
+
+    Attributes:
+        migration_state (google.cloud.datastore_admin_v1.types.MigrationState):
+            The current state of migration from Cloud
+            Datastore to Cloud Firestore in Datastore mode.
+        migration_step (google.cloud.datastore_admin_v1.types.MigrationStep):
+            The current step of migration from Cloud
+            Datastore to Cloud Firestore in Datastore mode.
+    """
+
+    migration_state = proto.Field(proto.ENUM, number=1, enum=migration.MigrationState,)
+    migration_step = proto.Field(proto.ENUM, number=2, enum=migration.MigrationStep,)
+
+
 __all__ = tuple(sorted(__protobuf__.manifest))
