@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,23 +14,25 @@
 # limitations under the License.
 #
 from collections import OrderedDict
-from distutils import util
 import os
 import re
 from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core import client_options as client_options_lib  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core import client_options as client_options_lib
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport import mtls  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 from google.auth.exceptions import MutualTLSChannelError  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
-OptionalRetry = Union[retries.Retry, object]
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
 
 from google.cloud.datastore_v1.types import datastore
 from google.cloud.datastore_v1.types import entity
@@ -222,6 +224,73 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
         m = re.match(r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)$", path)
         return m.groupdict() if m else {}
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[client_options_lib.ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        if client_options is None:
+            client_options = client_options_lib.ClientOptions()
+        use_client_cert = os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false")
+        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
+        if use_client_cert not in ("true", "false"):
+            raise ValueError(
+                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
+            )
+        if use_mtls_endpoint not in ("auto", "never", "always"):
+            raise MutualTLSChannelError(
+                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
+            )
+
+        # Figure out the client cert source to use.
+        client_cert_source = None
+        if use_client_cert == "true":
+            if client_options.client_cert_source:
+                client_cert_source = client_options.client_cert_source
+            elif mtls.has_default_client_cert_source():
+                client_cert_source = mtls.default_client_cert_source()
+
+        # Figure out which api endpoint to use.
+        if client_options.api_endpoint is not None:
+            api_endpoint = client_options.api_endpoint
+        elif use_mtls_endpoint == "always" or (
+            use_mtls_endpoint == "auto" and client_cert_source
+        ):
+            api_endpoint = cls.DEFAULT_MTLS_ENDPOINT
+        else:
+            api_endpoint = cls.DEFAULT_ENDPOINT
+
+        return api_endpoint, client_cert_source
+
     def __init__(
         self,
         *,
@@ -272,50 +341,22 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
         if client_options is None:
             client_options = client_options_lib.ClientOptions()
 
-        # Create SSL credentials for mutual TLS if needed.
-        use_client_cert = bool(
-            util.strtobool(os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false"))
+        api_endpoint, client_cert_source_func = self.get_mtls_endpoint_and_cert_source(
+            client_options
         )
 
-        client_cert_source_func = None
-        is_mtls = False
-        if use_client_cert:
-            if client_options.client_cert_source:
-                is_mtls = True
-                client_cert_source_func = client_options.client_cert_source
-            else:
-                is_mtls = mtls.has_default_client_cert_source()
-                if is_mtls:
-                    client_cert_source_func = mtls.default_client_cert_source()
-                else:
-                    client_cert_source_func = None
-
-        # Figure out which api endpoint to use.
-        if client_options.api_endpoint is not None:
-            api_endpoint = client_options.api_endpoint
-        else:
-            use_mtls_env = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
-            if use_mtls_env == "never":
-                api_endpoint = self.DEFAULT_ENDPOINT
-            elif use_mtls_env == "always":
-                api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-            elif use_mtls_env == "auto":
-                if is_mtls:
-                    api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-                else:
-                    api_endpoint = self.DEFAULT_ENDPOINT
-            else:
-                raise MutualTLSChannelError(
-                    "Unsupported GOOGLE_API_USE_MTLS_ENDPOINT value. Accepted "
-                    "values: never, auto, always"
-                )
+        api_key_value = getattr(client_options, "api_key", None)
+        if api_key_value and credentials:
+            raise ValueError(
+                "client_options.api_key and credentials are mutually exclusive"
+            )
 
         # Save or instantiate the transport.
         # Ordinarily, we provide the transport, but allowing a custom transport
         # instance provides an extensibility point for unusual situations.
         if isinstance(transport, DatastoreTransport):
             # transport is a DatastoreTransport instance.
-            if credentials or client_options.credentials_file:
+            if credentials or client_options.credentials_file or api_key_value:
                 raise ValueError(
                     "When providing a transport instance, "
                     "provide its credentials directly."
@@ -327,6 +368,15 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
                 )
             self._transport = transport
         else:
+            import google.auth._default  # type: ignore
+
+            if api_key_value and hasattr(
+                google.auth._default, "get_api_key_credentials"
+            ):
+                credentials = google.auth._default.get_api_key_credentials(
+                    api_key_value
+                )
+
             Transport = type(self).get_transport_class(transport)
             self._transport = Transport(
                 credentials=credentials,
@@ -352,6 +402,25 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
     ) -> datastore.LookupResponse:
         r"""Looks up entities by key.
 
+        .. code-block:: python
+
+            from google.cloud import datastore_v1
+
+            def sample_lookup():
+                # Create a client
+                client = datastore_v1.DatastoreClient()
+
+                # Initialize request argument(s)
+                request = datastore_v1.LookupRequest(
+                    project_id="project_id_value",
+                )
+
+                # Make the request
+                response = client.lookup(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_v1.types.LookupRequest, dict]):
                 The request object. The request for
@@ -388,7 +457,7 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([project_id, read_options, keys])
         if request is not None and has_flattened_params:
@@ -432,6 +501,25 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
     ) -> datastore.RunQueryResponse:
         r"""Queries for entities.
 
+        .. code-block:: python
+
+            from google.cloud import datastore_v1
+
+            def sample_run_query():
+                # Create a client
+                client = datastore_v1.DatastoreClient()
+
+                # Initialize request argument(s)
+                request = datastore_v1.RunQueryRequest(
+                    project_id="project_id_value",
+                )
+
+                # Make the request
+                response = client.run_query(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_v1.types.RunQueryRequest, dict]):
                 The request object. The request for
@@ -477,6 +565,25 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
     ) -> datastore.BeginTransactionResponse:
         r"""Begins a new transaction.
 
+        .. code-block:: python
+
+            from google.cloud import datastore_v1
+
+            def sample_begin_transaction():
+                # Create a client
+                client = datastore_v1.DatastoreClient()
+
+                # Initialize request argument(s)
+                request = datastore_v1.BeginTransactionRequest(
+                    project_id="project_id_value",
+                )
+
+                # Make the request
+                response = client.begin_transaction(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_v1.types.BeginTransactionRequest, dict]):
                 The request object. The request for
@@ -501,7 +608,7 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([project_id])
         if request is not None and has_flattened_params:
@@ -546,6 +653,27 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
         r"""Commits a transaction, optionally creating, deleting
         or modifying some entities.
 
+
+        .. code-block:: python
+
+            from google.cloud import datastore_v1
+
+            def sample_commit():
+                # Create a client
+                client = datastore_v1.DatastoreClient()
+
+                # Initialize request argument(s)
+                request = datastore_v1.CommitRequest(
+                    transaction=b'transaction_blob',
+                    project_id="project_id_value",
+                )
+
+                # Make the request
+                response = client.commit(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_v1.types.CommitRequest, dict]):
                 The request object. The request for
@@ -605,7 +733,7 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([project_id, mode, transaction, mutations])
         if request is not None and has_flattened_params:
@@ -653,6 +781,26 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
     ) -> datastore.RollbackResponse:
         r"""Rolls back a transaction.
 
+        .. code-block:: python
+
+            from google.cloud import datastore_v1
+
+            def sample_rollback():
+                # Create a client
+                client = datastore_v1.DatastoreClient()
+
+                # Initialize request argument(s)
+                request = datastore_v1.RollbackRequest(
+                    project_id="project_id_value",
+                    transaction=b'transaction_blob',
+                )
+
+                # Make the request
+                response = client.rollback(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_v1.types.RollbackRequest, dict]):
                 The request object. The request for
@@ -685,7 +833,7 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([project_id, transaction])
         if request is not None and has_flattened_params:
@@ -730,6 +878,26 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
         r"""Allocates IDs for the given keys, which is useful for
         referencing an entity before it is inserted.
 
+
+        .. code-block:: python
+
+            from google.cloud import datastore_v1
+
+            def sample_allocate_ids():
+                # Create a client
+                client = datastore_v1.DatastoreClient()
+
+                # Initialize request argument(s)
+                request = datastore_v1.AllocateIdsRequest(
+                    project_id="project_id_value",
+                )
+
+                # Make the request
+                response = client.allocate_ids(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_v1.types.AllocateIdsRequest, dict]):
                 The request object. The request for
@@ -763,7 +931,7 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([project_id, keys])
         if request is not None and has_flattened_params:
@@ -805,8 +973,28 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> datastore.ReserveIdsResponse:
-        r"""Prevents the supplied keys' IDs from being auto-
-        llocated by Cloud Datastore.
+        r"""Prevents the supplied keys' IDs from being
+        auto-allocated by Cloud Datastore.
+
+
+        .. code-block:: python
+
+            from google.cloud import datastore_v1
+
+            def sample_reserve_ids():
+                # Create a client
+                client = datastore_v1.DatastoreClient()
+
+                # Initialize request argument(s)
+                request = datastore_v1.ReserveIdsRequest(
+                    project_id="project_id_value",
+                )
+
+                # Make the request
+                response = client.reserve_ids(request=request)
+
+                # Handle the response
+                print(response)
 
         Args:
             request (Union[google.cloud.datastore_v1.types.ReserveIdsRequest, dict]):
@@ -840,7 +1028,7 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([project_id, keys])
         if request is not None and has_flattened_params:
