@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,8 +16,8 @@
 import warnings
 from typing import Callable, Dict, Optional, Sequence, Tuple, Union
 
-from google.api_core import grpc_helpers  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
+from google.api_core import grpc_helpers
+from google.api_core import gapic_v1
 import google.auth  # type: ignore
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
@@ -164,8 +164,11 @@ class DatastoreGrpcTransport(DatastoreTransport):
         if not self._grpc_channel:
             self._grpc_channel = type(self).create_channel(
                 self._host,
+                # use the credentials which are saved
                 credentials=self._credentials,
-                credentials_file=credentials_file,
+                # Set ``credentials_file`` to ``None`` here as
+                # the credentials that we saved earlier should be used.
+                credentials_file=None,
                 scopes=self._scopes,
                 ssl_credentials=self._ssl_channel_credentials,
                 quota_project_id=quota_project_id,
@@ -393,8 +396,8 @@ class DatastoreGrpcTransport(DatastoreTransport):
     ) -> Callable[[datastore.ReserveIdsRequest], datastore.ReserveIdsResponse]:
         r"""Return a callable for the reserve ids method over gRPC.
 
-        Prevents the supplied keys' IDs from being auto-
-        llocated by Cloud Datastore.
+        Prevents the supplied keys' IDs from being
+        auto-allocated by Cloud Datastore.
 
         Returns:
             Callable[[~.ReserveIdsRequest],
