@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -238,20 +238,20 @@ def test_datastore_client_client_options(client_class, transport_class, transpor
     # unsupported value.
     with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "Unsupported"}):
         with pytest.raises(MutualTLSChannelError):
-            client = client_class()
+            client = client_class(transport=transport_name)
 
     # Check the case GOOGLE_API_USE_CLIENT_CERTIFICATE has unsupported value.
     with mock.patch.dict(
         os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "Unsupported"}
     ):
         with pytest.raises(ValueError):
-            client = client_class()
+            client = client_class(transport=transport_name)
 
     # Check the case quota_project_id is provided
     options = client_options.ClientOptions(quota_project_id="octopus")
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(transport=transport_name, client_options=options)
+        client = client_class(client_options=options, transport=transport_name)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file=None,
@@ -308,7 +308,7 @@ def test_datastore_client_mtls_env_auto(
         )
         with mock.patch.object(transport_class, "__init__") as patched:
             patched.return_value = None
-            client = client_class(transport=transport_name, client_options=options)
+            client = client_class(client_options=options, transport=transport_name)
 
             if use_client_cert_env == "false":
                 expected_client_cert_source = None
@@ -385,6 +385,83 @@ def test_datastore_client_mtls_env_auto(
                 )
 
 
+@pytest.mark.parametrize("client_class", [DatastoreClient, DatastoreAsyncClient])
+@mock.patch.object(
+    DatastoreClient, "DEFAULT_ENDPOINT", modify_default_endpoint(DatastoreClient)
+)
+@mock.patch.object(
+    DatastoreAsyncClient,
+    "DEFAULT_ENDPOINT",
+    modify_default_endpoint(DatastoreAsyncClient),
+)
+def test_datastore_client_get_mtls_endpoint_and_cert_source(client_class):
+    mock_client_cert_source = mock.Mock()
+
+    # Test the case GOOGLE_API_USE_CLIENT_CERTIFICATE is "true".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
+        mock_api_endpoint = "foo"
+        options = client_options.ClientOptions(
+            client_cert_source=mock_client_cert_source, api_endpoint=mock_api_endpoint
+        )
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source(
+            options
+        )
+        assert api_endpoint == mock_api_endpoint
+        assert cert_source == mock_client_cert_source
+
+    # Test the case GOOGLE_API_USE_CLIENT_CERTIFICATE is "false".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "false"}):
+        mock_client_cert_source = mock.Mock()
+        mock_api_endpoint = "foo"
+        options = client_options.ClientOptions(
+            client_cert_source=mock_client_cert_source, api_endpoint=mock_api_endpoint
+        )
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source(
+            options
+        )
+        assert api_endpoint == mock_api_endpoint
+        assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "never".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
+        assert api_endpoint == client_class.DEFAULT_ENDPOINT
+        assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "always".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "always"}):
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
+        assert api_endpoint == client_class.DEFAULT_MTLS_ENDPOINT
+        assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "auto" and default cert doesn't exist.
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
+        with mock.patch(
+            "google.auth.transport.mtls.has_default_client_cert_source",
+            return_value=False,
+        ):
+            api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
+            assert api_endpoint == client_class.DEFAULT_ENDPOINT
+            assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "auto" and default cert exists.
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
+        with mock.patch(
+            "google.auth.transport.mtls.has_default_client_cert_source",
+            return_value=True,
+        ):
+            with mock.patch(
+                "google.auth.transport.mtls.default_client_cert_source",
+                return_value=mock_client_cert_source,
+            ):
+                (
+                    api_endpoint,
+                    cert_source,
+                ) = client_class.get_mtls_endpoint_and_cert_source()
+                assert api_endpoint == client_class.DEFAULT_MTLS_ENDPOINT
+                assert cert_source == mock_client_cert_source
+
+
 @pytest.mark.parametrize(
     "client_class,transport_class,transport_name",
     [
@@ -403,7 +480,7 @@ def test_datastore_client_client_options_scopes(
     options = client_options.ClientOptions(scopes=["1", "2"],)
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(transport=transport_name, client_options=options)
+        client = client_class(client_options=options, transport=transport_name)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file=None,
@@ -417,24 +494,26 @@ def test_datastore_client_client_options_scopes(
 
 
 @pytest.mark.parametrize(
-    "client_class,transport_class,transport_name",
+    "client_class,transport_class,transport_name,grpc_helpers",
     [
-        (DatastoreClient, transports.DatastoreGrpcTransport, "grpc"),
+        (DatastoreClient, transports.DatastoreGrpcTransport, "grpc", grpc_helpers),
         (
             DatastoreAsyncClient,
             transports.DatastoreGrpcAsyncIOTransport,
             "grpc_asyncio",
+            grpc_helpers_async,
         ),
     ],
 )
 def test_datastore_client_client_options_credentials_file(
-    client_class, transport_class, transport_name
+    client_class, transport_class, transport_name, grpc_helpers
 ):
     # Check the case credentials file is provided.
     options = client_options.ClientOptions(credentials_file="credentials.json")
+
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(transport=transport_name, client_options=options)
+        client = client_class(client_options=options, transport=transport_name)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file="credentials.json",
@@ -465,7 +544,72 @@ def test_datastore_client_client_options_from_dict():
         )
 
 
-def test_lookup(transport: str = "grpc", request_type=datastore.LookupRequest):
+@pytest.mark.parametrize(
+    "client_class,transport_class,transport_name,grpc_helpers",
+    [
+        (DatastoreClient, transports.DatastoreGrpcTransport, "grpc", grpc_helpers),
+        (
+            DatastoreAsyncClient,
+            transports.DatastoreGrpcAsyncIOTransport,
+            "grpc_asyncio",
+            grpc_helpers_async,
+        ),
+    ],
+)
+def test_datastore_client_create_channel_credentials_file(
+    client_class, transport_class, transport_name, grpc_helpers
+):
+    # Check the case credentials file is provided.
+    options = client_options.ClientOptions(credentials_file="credentials.json")
+
+    with mock.patch.object(transport_class, "__init__") as patched:
+        patched.return_value = None
+        client = client_class(client_options=options, transport=transport_name)
+        patched.assert_called_once_with(
+            credentials=None,
+            credentials_file="credentials.json",
+            host=client.DEFAULT_ENDPOINT,
+            scopes=None,
+            client_cert_source_for_mtls=None,
+            quota_project_id=None,
+            client_info=transports.base.DEFAULT_CLIENT_INFO,
+            always_use_jwt_access=True,
+        )
+
+    # test that the credentials from file are saved and used as the credentials.
+    with mock.patch.object(
+        google.auth, "load_credentials_from_file", autospec=True
+    ) as load_creds, mock.patch.object(
+        google.auth, "default", autospec=True
+    ) as adc, mock.patch.object(
+        grpc_helpers, "create_channel"
+    ) as create_channel:
+        creds = ga_credentials.AnonymousCredentials()
+        file_creds = ga_credentials.AnonymousCredentials()
+        load_creds.return_value = (file_creds, None)
+        adc.return_value = (creds, None)
+        client = client_class(client_options=options, transport=transport_name)
+        create_channel.assert_called_with(
+            "datastore.googleapis.com:443",
+            credentials=file_creds,
+            credentials_file=None,
+            quota_project_id=None,
+            default_scopes=(
+                "https://www.googleapis.com/auth/cloud-platform",
+                "https://www.googleapis.com/auth/datastore",
+            ),
+            scopes=None,
+            default_host="datastore.googleapis.com",
+            ssl_credentials=None,
+            options=[
+                ("grpc.max_send_message_length", -1),
+                ("grpc.max_receive_message_length", -1),
+            ],
+        )
+
+
+@pytest.mark.parametrize("request_type", [datastore.LookupRequest, dict,])
+def test_lookup(request_type, transport: str = "grpc"):
     client = DatastoreClient(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -489,10 +633,6 @@ def test_lookup(transport: str = "grpc", request_type=datastore.LookupRequest):
     assert isinstance(response, datastore.LookupResponse)
 
 
-def test_lookup_from_dict():
-    test_lookup(request_type=dict)
-
-
 def test_lookup_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -567,13 +707,19 @@ def test_lookup_flattened():
         # request object values.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-        assert args[0].project_id == "project_id_value"
-        assert args[0].read_options == datastore.ReadOptions(
+        arg = args[0].project_id
+        mock_val = "project_id_value"
+        assert arg == mock_val
+        arg = args[0].read_options
+        mock_val = datastore.ReadOptions(
             read_consistency=datastore.ReadOptions.ReadConsistency.STRONG
         )
-        assert args[0].keys == [
+        assert arg == mock_val
+        arg = args[0].keys
+        mock_val = [
             entity.Key(partition_id=entity.PartitionId(project_id="project_id_value"))
         ]
+        assert arg == mock_val
 
 
 def test_lookup_flattened_error():
@@ -626,13 +772,19 @@ async def test_lookup_flattened_async():
         # request object values.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-        assert args[0].project_id == "project_id_value"
-        assert args[0].read_options == datastore.ReadOptions(
+        arg = args[0].project_id
+        mock_val = "project_id_value"
+        assert arg == mock_val
+        arg = args[0].read_options
+        mock_val = datastore.ReadOptions(
             read_consistency=datastore.ReadOptions.ReadConsistency.STRONG
         )
-        assert args[0].keys == [
+        assert arg == mock_val
+        arg = args[0].keys
+        mock_val = [
             entity.Key(partition_id=entity.PartitionId(project_id="project_id_value"))
         ]
+        assert arg == mock_val
 
 
 @pytest.mark.asyncio
@@ -656,7 +808,8 @@ async def test_lookup_flattened_error_async():
         )
 
 
-def test_run_query(transport: str = "grpc", request_type=datastore.RunQueryRequest):
+@pytest.mark.parametrize("request_type", [datastore.RunQueryRequest, dict,])
+def test_run_query(request_type, transport: str = "grpc"):
     client = DatastoreClient(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -680,10 +833,6 @@ def test_run_query(transport: str = "grpc", request_type=datastore.RunQueryReque
     assert isinstance(response, datastore.RunQueryResponse)
 
 
-def test_run_query_from_dict():
-    test_run_query(request_type=dict)
-
-
 def test_run_query_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -733,9 +882,8 @@ async def test_run_query_async_from_dict():
     await test_run_query_async(request_type=dict)
 
 
-def test_begin_transaction(
-    transport: str = "grpc", request_type=datastore.BeginTransactionRequest
-):
+@pytest.mark.parametrize("request_type", [datastore.BeginTransactionRequest, dict,])
+def test_begin_transaction(request_type, transport: str = "grpc"):
     client = DatastoreClient(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -764,10 +912,6 @@ def test_begin_transaction(
     assert response.transaction == b"transaction_blob"
 
 
-def test_begin_transaction_from_dict():
-    test_begin_transaction(request_type=dict)
-
-
 def test_begin_transaction_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -839,7 +983,9 @@ def test_begin_transaction_flattened():
         # request object values.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-        assert args[0].project_id == "project_id_value"
+        arg = args[0].project_id
+        mock_val = "project_id_value"
+        assert arg == mock_val
 
 
 def test_begin_transaction_flattened_error():
@@ -875,7 +1021,9 @@ async def test_begin_transaction_flattened_async():
         # request object values.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-        assert args[0].project_id == "project_id_value"
+        arg = args[0].project_id
+        mock_val = "project_id_value"
+        assert arg == mock_val
 
 
 @pytest.mark.asyncio
@@ -890,7 +1038,8 @@ async def test_begin_transaction_flattened_error_async():
         )
 
 
-def test_commit(transport: str = "grpc", request_type=datastore.CommitRequest):
+@pytest.mark.parametrize("request_type", [datastore.CommitRequest, dict,])
+def test_commit(request_type, transport: str = "grpc"):
     client = DatastoreClient(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -915,10 +1064,6 @@ def test_commit(transport: str = "grpc", request_type=datastore.CommitRequest):
     assert response.index_updates == 1389
 
 
-def test_commit_from_dict():
-    test_commit(request_type=dict)
-
-
 def test_commit_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -999,9 +1144,14 @@ def test_commit_flattened():
         # request object values.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-        assert args[0].project_id == "project_id_value"
-        assert args[0].mode == datastore.CommitRequest.Mode.TRANSACTIONAL
-        assert args[0].mutations == [
+        arg = args[0].project_id
+        mock_val = "project_id_value"
+        assert arg == mock_val
+        arg = args[0].mode
+        mock_val = datastore.CommitRequest.Mode.TRANSACTIONAL
+        assert arg == mock_val
+        arg = args[0].mutations
+        mock_val = [
             datastore.Mutation(
                 insert=entity.Entity(
                     key=entity.Key(
@@ -1010,6 +1160,7 @@ def test_commit_flattened():
                 )
             )
         ]
+        assert arg == mock_val
         assert args[0].transaction == b"transaction_blob"
 
 
@@ -1073,9 +1224,14 @@ async def test_commit_flattened_async():
         # request object values.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-        assert args[0].project_id == "project_id_value"
-        assert args[0].mode == datastore.CommitRequest.Mode.TRANSACTIONAL
-        assert args[0].mutations == [
+        arg = args[0].project_id
+        mock_val = "project_id_value"
+        assert arg == mock_val
+        arg = args[0].mode
+        mock_val = datastore.CommitRequest.Mode.TRANSACTIONAL
+        assert arg == mock_val
+        arg = args[0].mutations
+        mock_val = [
             datastore.Mutation(
                 insert=entity.Entity(
                     key=entity.Key(
@@ -1084,6 +1240,7 @@ async def test_commit_flattened_async():
                 )
             )
         ]
+        assert arg == mock_val
         assert args[0].transaction == b"transaction_blob"
 
 
@@ -1113,7 +1270,8 @@ async def test_commit_flattened_error_async():
         )
 
 
-def test_rollback(transport: str = "grpc", request_type=datastore.RollbackRequest):
+@pytest.mark.parametrize("request_type", [datastore.RollbackRequest, dict,])
+def test_rollback(request_type, transport: str = "grpc"):
     client = DatastoreClient(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -1137,10 +1295,6 @@ def test_rollback(transport: str = "grpc", request_type=datastore.RollbackReques
     assert isinstance(response, datastore.RollbackResponse)
 
 
-def test_rollback_from_dict():
-    test_rollback(request_type=dict)
-
-
 def test_rollback_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -1207,8 +1361,12 @@ def test_rollback_flattened():
         # request object values.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-        assert args[0].project_id == "project_id_value"
-        assert args[0].transaction == b"transaction_blob"
+        arg = args[0].project_id
+        mock_val = "project_id_value"
+        assert arg == mock_val
+        arg = args[0].transaction
+        mock_val = b"transaction_blob"
+        assert arg == mock_val
 
 
 def test_rollback_flattened_error():
@@ -1246,8 +1404,12 @@ async def test_rollback_flattened_async():
         # request object values.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-        assert args[0].project_id == "project_id_value"
-        assert args[0].transaction == b"transaction_blob"
+        arg = args[0].project_id
+        mock_val = "project_id_value"
+        assert arg == mock_val
+        arg = args[0].transaction
+        mock_val = b"transaction_blob"
+        assert arg == mock_val
 
 
 @pytest.mark.asyncio
@@ -1264,9 +1426,8 @@ async def test_rollback_flattened_error_async():
         )
 
 
-def test_allocate_ids(
-    transport: str = "grpc", request_type=datastore.AllocateIdsRequest
-):
+@pytest.mark.parametrize("request_type", [datastore.AllocateIdsRequest, dict,])
+def test_allocate_ids(request_type, transport: str = "grpc"):
     client = DatastoreClient(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -1290,10 +1451,6 @@ def test_allocate_ids(
     assert isinstance(response, datastore.AllocateIdsResponse)
 
 
-def test_allocate_ids_from_dict():
-    test_allocate_ids(request_type=dict)
-
-
 def test_allocate_ids_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -1365,10 +1522,14 @@ def test_allocate_ids_flattened():
         # request object values.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-        assert args[0].project_id == "project_id_value"
-        assert args[0].keys == [
+        arg = args[0].project_id
+        mock_val = "project_id_value"
+        assert arg == mock_val
+        arg = args[0].keys
+        mock_val = [
             entity.Key(partition_id=entity.PartitionId(project_id="project_id_value"))
         ]
+        assert arg == mock_val
 
 
 def test_allocate_ids_flattened_error():
@@ -1415,10 +1576,14 @@ async def test_allocate_ids_flattened_async():
         # request object values.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-        assert args[0].project_id == "project_id_value"
-        assert args[0].keys == [
+        arg = args[0].project_id
+        mock_val = "project_id_value"
+        assert arg == mock_val
+        arg = args[0].keys
+        mock_val = [
             entity.Key(partition_id=entity.PartitionId(project_id="project_id_value"))
         ]
+        assert arg == mock_val
 
 
 @pytest.mark.asyncio
@@ -1439,7 +1604,8 @@ async def test_allocate_ids_flattened_error_async():
         )
 
 
-def test_reserve_ids(transport: str = "grpc", request_type=datastore.ReserveIdsRequest):
+@pytest.mark.parametrize("request_type", [datastore.ReserveIdsRequest, dict,])
+def test_reserve_ids(request_type, transport: str = "grpc"):
     client = DatastoreClient(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -1463,10 +1629,6 @@ def test_reserve_ids(transport: str = "grpc", request_type=datastore.ReserveIdsR
     assert isinstance(response, datastore.ReserveIdsResponse)
 
 
-def test_reserve_ids_from_dict():
-    test_reserve_ids(request_type=dict)
-
-
 def test_reserve_ids_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -1538,10 +1700,14 @@ def test_reserve_ids_flattened():
         # request object values.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-        assert args[0].project_id == "project_id_value"
-        assert args[0].keys == [
+        arg = args[0].project_id
+        mock_val = "project_id_value"
+        assert arg == mock_val
+        arg = args[0].keys
+        mock_val = [
             entity.Key(partition_id=entity.PartitionId(project_id="project_id_value"))
         ]
+        assert arg == mock_val
 
 
 def test_reserve_ids_flattened_error():
@@ -1588,10 +1754,14 @@ async def test_reserve_ids_flattened_async():
         # request object values.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-        assert args[0].project_id == "project_id_value"
-        assert args[0].keys == [
+        arg = args[0].project_id
+        mock_val = "project_id_value"
+        assert arg == mock_val
+        arg = args[0].keys
+        mock_val = [
             entity.Key(partition_id=entity.PartitionId(project_id="project_id_value"))
         ]
+        assert arg == mock_val
 
 
 @pytest.mark.asyncio
@@ -1632,6 +1802,23 @@ def test_credentials_transport_error():
             transport=transport,
         )
 
+    # It is an error to provide an api_key and a transport instance.
+    transport = transports.DatastoreGrpcTransport(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
+    options = client_options.ClientOptions()
+    options.api_key = "api_key"
+    with pytest.raises(ValueError):
+        client = DatastoreClient(client_options=options, transport=transport,)
+
+    # It is an error to provide an api_key and a credential.
+    options = mock.Mock()
+    options.api_key = "api_key"
+    with pytest.raises(ValueError):
+        client = DatastoreClient(
+            client_options=options, credentials=ga_credentials.AnonymousCredentials()
+        )
+
     # It is an error to provide scopes and a transport instance.
     transport = transports.DatastoreGrpcTransport(
         credentials=ga_credentials.AnonymousCredentials(),
@@ -2103,7 +2290,7 @@ def test_parse_common_location_path():
     assert expected == actual
 
 
-def test_client_withDEFAULT_CLIENT_INFO():
+def test_client_with_default_client_info():
     client_info = gapic_v1.client_info.ClientInfo()
 
     with mock.patch.object(
@@ -2168,3 +2355,33 @@ def test_client_ctx():
             with client:
                 pass
             close.assert_called()
+
+
+@pytest.mark.parametrize(
+    "client_class,transport_class",
+    [
+        (DatastoreClient, transports.DatastoreGrpcTransport),
+        (DatastoreAsyncClient, transports.DatastoreGrpcAsyncIOTransport),
+    ],
+)
+def test_api_key_credentials(client_class, transport_class):
+    with mock.patch.object(
+        google.auth._default, "get_api_key_credentials", create=True
+    ) as get_api_key_credentials:
+        mock_cred = mock.Mock()
+        get_api_key_credentials.return_value = mock_cred
+        options = client_options.ClientOptions()
+        options.api_key = "api_key"
+        with mock.patch.object(transport_class, "__init__") as patched:
+            patched.return_value = None
+            client = client_class(client_options=options)
+            patched.assert_called_once_with(
+                credentials=mock_cred,
+                credentials_file=None,
+                host=client.DEFAULT_ENDPOINT,
+                scopes=None,
+                client_cert_source_for_mtls=None,
+                quota_project_id=None,
+                client_info=transports.base.DEFAULT_CLIENT_INFO,
+                always_use_jwt_access=True,
+            )
